"""
    test_domonic
    ~~~~~~~~~~~~
    unit tests for css
"""

import unittest

from domonic.dom import *
from domonic.html import *
from domonic.style import *

# import requests
# from mock import patch



class TestCase(unittest.TestCase):

    def test_domonic_css(self):

        test = div("huh?", _style="alignContent: center;")  # TODO - should be parsing these styles
        print(test.style.alignContent)
        test.style.alignContent = 'flex-start'
        print(test.style.alignContent)
        print(str(test))
        assert test.style.alignContent == 'flex-start'
        # assert str(test) == '<div style="align-content: flex-start;">huh?</div>'

        atag = a("linky", _href="https://eventual.technology", _style="alignContent: center;")
        print(atag.style.alignContent)

        sometag = div("asdfasdf", _id="test")
        print(sometag.style)
        sometag.style.alignContent = None
        print(sometag.style.alignContent)

        sometag.style.backgroundColor = "black"
        sometag.style.fontSize = "12px"

        # huh = document.createAttribute("test")
        # huh.value = "wtf"
        # sometag.setAttributeNode(huh)
        print(sometag.style.fontSize)
        print(sometag)

        # dom.select('#test' ).dostuff() # TODO -
        # print(sometag.style)
        # print(sometag.tagName)
        # s = Style()
        # print(sometag)

    # create some failing tests

    # def test_css_style_declaration(self):
        # styleObj = document.styleSheets[0].cssRules[0].style
        # print(styleObj.cssText)

    # def test_css_style_rules(self):
        # myRules = document.styleSheets[0].cssRules # Returns a CSSRuleList
        # print(myRules)

        # myRules = document.styleSheets[0].cssRules
        # print(myRules[0]); # a CSSStyleRule representing the h1.

    #def test_css_styledpropertymap(self):
        # pass

    def test_paser(self):

        somecss = '''
            :host {
                display: block;
            }

            div {
                color: red;
            }

            .class1 {
                color: blue;
            }

            .class1.class2 span {
                color: green;
            }

            .class1[attr1=\"value1\"] {
                color: yellow;
            }

            [attr1=\"value1\"] {
                color: yellow;
            }
        '''

        from domonic.style import CSSParser

        ss = CSSStyleSheet()
        p = CSSParser.parseFromString(ss, somecss)

        print("SHEET:", ss.rules)
        print("PARSER:", p)

        for r in p:
            print("bo:", r.selectorText)
            print(r.style.cssText)
            # print(r.style)
            # print(r.parentRule)

        cssStyleSheet: CSSStyleSheet = CSSStyleSheet()
        cssStyleSheet.insertRule('div { background-color: green }')
        cssStyleSheet.insertRule('span { background-color: green }')
        cssStyleSheet.insertRule('div { background-color: green }')

        # print(cssStyleSheet.cssRules)
        # print(cssStyleSheet.cssRules.length)
        # print(cssStyleSheet.cssRules[0].selectorText)
        # print(cssStyleSheet.cssRules[0].style.cssText)
        assert cssStyleSheet.cssRules.length == 3
        assert cssStyleSheet.cssRules[0].selectorText == 'div'
        assert cssStyleSheet.cssRules[0].style.cssText == 'background-color: green'

        # cssStyleSheet.insertRule('background-color: green');
        # DOMException('Invalid CSS rule.', DOMExceptionNameEnum.hierarchyRequestError)
        somecss = '''
            :host {
                display: flex;
                overflow: hidden;
                width: 100%;
            }
            .container {
                flex-grow: 1;
                display: flex;
                flex-direction: column;
                overflow: hidden;
            }
            @media screen and (max-width: 36rem) {
                .container {
                    height: 0.5rem;
                    animation: keyframes2 2s linear infinite;
                }
            }
            @keyframes keyframes1 {
                from {
                    transform: rotate(0deg);
                }
                to {
                    transform: rotate(360deg);
                }
            }
            @keyframes keyframes2 {
                0% {
                    transform: rotate(0deg);
                }
                100% {
                    transform: rotate(360deg);
                }
            }
        '''

        cssStyleSheet = CSSStyleSheet()
        cssRules = CSSParser.parseFromString(cssStyleSheet, somecss)

        assert len(cssRules) == 5

        # CSSStyleRule
        assert cssRules[0].parentRule == None
        assert cssRules[0].parentStyleSheet == cssStyleSheet
        assert cssRules[0].selectorText == ':host'
        print('>>>>>>>>>>>>>>>>>>>>>', cssRules[0].cssText)
        # assert cssRules[0].cssText == ':host { display: flex; overflow: hidden; width: 100%; }'
        assert cssRules[0].style.parentRule == cssRules[0]
        # assert cssRules[0].style.length == 3
        # assert cssRules[0].style[0] == 'display'
        # assert cssRules[0].style[1] == 'overflow'
        # assert cssRules[0].style[2] == 'width'
        # assert cssRules[0].style['display'] == 'flex'
        # assert cssRules[0].style['overflow'] == 'hidden'
        # assert cssRules[0].style['width'] == '100%'
        assert cssRules[0].style.cssText == 'display: flex; overflow: hidden; width: 100%;'

        # CSSStyleRule
        assert cssRules[1].parentRule == None
        assert cssRules[1].parentStyleSheet == cssStyleSheet
        assert cssRules[1].selectorText == '.container'
        # assert cssRules[1].cssText == '.container { flex-grow: 1; display: flex; flex-direction: column; overflow: hidden; }'
        # assert cssRules[1].style.length == 4
        assert cssRules[1].style.parentRule == cssRules[1]
        # assert cssRules[1].style[0] == 'flex-grow'
        # assert cssRules[1].style[1] == 'display'
        # assert cssRules[1].style[2] == 'flex-direction'
        # assert cssRules[1].style[3] == 'overflow'
        # assert cssRules[1].style['flexGrow'] == '1'
        # assert cssRules[1].style['display'] == 'flex'
        # assert cssRules[1].style['flexDirection'] == 'column'
        # assert cssRules[1].style['overflow'] == 'hidden'
        assert cssRules[1].style.cssText == 'flex-grow: 1; display: flex; flex-direction: column; overflow: hidden;'

        # CSSMediaRule
        assert cssRules[2].parentRule == None
        assert cssRules[2].parentStyleSheet == cssStyleSheet
        assert cssRules[2].media.length == 1
        assert cssRules[2].media[0] == 'screen and (max-width: 36rem)'
        assert cssRules[2].media.mediaText == 'screen and (max-width: 36rem)'
        # assert cssRules[2].cssText == '@media screen and (max-width: 36rem) { .container { height: 0.5rem; animation: keyframes2 2s linear infinite; } }'
        # assert cssRules[2].cssRules.length == 1
        children1 = cssRules[2].cssRules
        assert children1[0].parentRule == cssRules[2]
        assert children1[0].parentStyleSheet == cssStyleSheet
        assert children1[0].selectorText == '.container'
        # assert children1[0].style.length == 2
        assert children1[0].style.parentRule == children1[0]
        # assert children1[0].style[0] == 'height'
        # assert children1[0].style[1] == 'animation'
        # assert children1[0].style['height'] == '0.5rem'
        # assert children1[0].style['animation'] == 'keyframes2 2s linear infinite'
        # assert children1[0].cssText == '.container { height: 0.5rem; animation: keyframes2 2s linear infinite; }'

        # CSSKeyframesRule
        assert cssRules[3].parentRule == None
        assert cssRules[3].parentStyleSheet == cssStyleSheet
        assert cssRules[3].name == 'keyframes1'
        print(cssRules[3].cssText)
        # assert cssRules[3].cssText == '@keyframes keyframes1 { from { transform: rotate(0deg); } to { transform: rotate(360deg); } }'
        # assert cssRules[3].cssRules.length == 2
        children2 = cssRules[3].cssRules
        assert children2[0].parentRule == cssRules[3]
        assert children2[0].parentStyleSheet == cssStyleSheet
        assert children2[0].keyText == 'from'
        # assert children2[0].style.length == 1
        assert children2[0].style.parentRule == children2[0]
        # assert children2[0].style[0] == 'transform'
        # assert children2[0].style['transform'] == 'rotate(0deg)'
        print(children2[0].cssText)
        # assert children2[0].cssText == 'from { transform: rotate(0deg); }'
        assert children2[1].parentRule == cssRules[3]
        assert children2[1].parentStyleSheet == cssStyleSheet
        assert children2[1].keyText == 'to'
        # assert children2[1].style.length == 1
        # assert children2[1].style[0] == 'transform'
        # assert children2[1].style['transform'] == 'rotate(360deg)'
        # assert children2[1].cssText == 'to { transform: rotate(360deg); }'

        # CSSKeyframesRule
        assert cssRules[4].parentRule == None
        assert cssRules[4].parentStyleSheet == cssStyleSheet
        assert cssRules[4].name == 'keyframes2'
        # assert cssRules[4].cssText == '@keyframes keyframes2 { 0% { transform: rotate(0deg); } 100% { transform: rotate(360deg); } }'
        # assert cssRules[4].cssRules.length == 2
        children3 = cssRules[4].cssRules
        assert children3[0].parentRule == cssRules[4]
        assert children3[0].parentStyleSheet == cssStyleSheet
        assert children3[0].keyText == '0%'
        # assert children3[0].style.length == 1
        assert children3[0].style.parentRule == children3[0]
        # assert children3[0].style[0] == 'transform'
        # assert children3[0].style['transform'] == 'rotate(0deg)'
        print(children3[0].cssText)
        # assert children3[0].cssText == '0% { transform: rotate(0deg); }'
        assert children3[1].parentRule == cssRules[4]
        assert children3[1].parentStyleSheet == cssStyleSheet
        assert children3[1].keyText == '100%'
        # assert children3[1].style.length == 1
        # assert children3[1].style[0] == 'transform'
        # assert children3[1].style['transform'] == 'rotate(360deg)'
        # assert children3[1].cssText == '100% { transform: rotate(360deg); }'


    def test_new_props(self):

        s = Style()
        s.all = 'border: 1px solid black;'
        assert s.all == 'border: 1px solid black;'

        s.alignmentBaseline = 'baseline'
        s.appearance = 'none'
        s.backdropFilter = 'blur(2px)'
        s.backgroundBlendMode = 'normal'
        s.backgroundPositionX = '0'
        s.backgroundPositionY = '0'
        s.backgroundRepeatX = 0
        s.backgroundRepeatY = 0
        s.baselineShift = '0'
        s.blockSize = '0'
        s.borderBlockEnd = '0'
        # s.borderBlockEndColor = None
        # s.borderBlockEndStyle = None
        # s.borderBlockEndWidth = None
        # s.borderBlockStart = None
        # s.borderBlockStartColor = None
        # s.borderBlockStartStyle = None
        # s.borderBlockStartWidth = None
        # s.borderInlineEnd = None
        # s.borderInlineEndColor = None
        # s.borderInlineEndStyle = None
        # s.borderInlineEndWidth = None
        # s.borderInlineStart = None
        # s.borderInlineStartColor = None
        # s.borderInlineStartStyle = None
        # s.borderInlineStartWidth = None
        # s.breakAfter = 'auto'
        # s.breakBefore = 'auto'
        # s.breakInside = 'auto'
        # s.bufferedRendering = 'auto'
        # s.caretColor = None
        # s.clipPath = None
        # s.clipRule = None
        # s.colorInterpolation = None
        # s.colorInterpolationFilters = None
        # s.colorRendering = None
        # s.colorScheme = None
        # s.contain = None
        # s.containIntrinsicSize = None
        # s.contentVisibility = None
        # s.counterSet = None
        # s.cx = None
        # s.cy = None
        # s.dominantBaseline = None
        # s.d = None
        # s.fill = None
        # s.fillOpacity = None
        # s.fillRule = None
        # s.fontDisplay = None
        # s.floodColor = None
        # s.floodOpacity = None
        # s.fontFeatureSettings = None
        # s.fontKerning = None
        # s.fontOpticalSizing = None
        # s.fontVariantCaps = None
        # s.fontVariantEastAsian = None
        # s.fontVariantLigatures = None
        # s.fontVariantNumeric = None
        # s.fontVariationSettings = None
        # s.gap = None
        # s.grid = None
        # s.gridArea = None
        # s.gridAutoColumns = None
        # s.gridAutoFlow = None
        # s.gridAutoRows = None
        # s.gridColumn = None
        # s.gridColumnEnd = None
        # s.gridColumnGap = None
        # s.gridColumnStart = None
        # s.gridGap = None
        # s.gridRow = None
        # s.gridRowEnd = None
        # s.gridRowGap = None
        # s.gridRowStart = None
        # s.gridTemplate = None
        # s.gridTemplateAreas = None
        # s.gridTemplateColumns = None
        # s.gridTemplateRows = None
        # s.imageRendering = None
        # s.inherits = None
        # s.initialValue = None
        # s.inlineSize = None
        # s.justifyItems = None
        # s.justifySelf = None
        # s.lightingColor = None
        # s.lineBreak = None
        # s.marginBlockEnd = None
        # s.marginBlockStart = None
        # s.marginInlineEnd = None
        # s.marginInlineStart = None
        # s.marker = None
        # s.markerEnd = None
        # s.markerMid = None
        # s.markerStart = None
        # s.mask = None
        # s.maskType = None
        # s.maxBlockSize = None
        # s.maxInlineSize = None
        # s.maxZoom = None
        # s.minBlockSize = None
        # s.minInlineSize = None
        # s.minZoom = None
        # s.mixBlendMode = None
        # s.objectFit = None
        # s.objectPosition = None
        # s.offset = None
        # s.offsetDistance = None
        # s.offsetPath = None
        # s.offsetRotate = None
        # s.orientation = None
        # s.overflow = None
        # s.overflowAnchor = None
        # s.overflowWrap = None
        # s.overscrollBehavior = None
        # s.overscrollBehaviorBlock = None
        # s.overscrollBehaviorInline = None
        # s.overscrollBehaviorX = None
        # s.overscrollBehaviorY = None
        # s.paddingBlockEnd = None
        # s.paddingBlockStart = None
        # s.paddingInlineEnd = None
        # s.paddingInlineStart = None
        # s.page = None
        # s.pageOrientation = None
        # s.paintOrder = None
        # s.perspective = None
        # s.perspectiveOrigin = None
        # s.placeContent = None
        # s.placeItems = None
        # s.placeSelf = None
        # s.pointerEvents = None
        # s.r = None
        # s.rowGap = None
        # s.rubyPosition = None
        # s.rx = None
        # s.ry = None
        # s.scrollBehavior = None
        # s.scrollMargin = None
        # s.scrollMarginBlock = None
        # s.scrollMarginBlockEnd = None
        # s.scrollMarginBlockStart = None
        # s.scrollMarginBottom = None
        # s.scrollMarginInline = None
        # s.scrollMarginInlineEnd = None
        # s.scrollMarginInlineStart = None
        # s.scrollMarginLeft = None
        # s.scrollMarginRight = None
        # s.scrollMarginTop = None
        # s.scrollPadding = None
        # s.scrollPaddingBlock = None
        # s.scrollPaddingBlockEnd = None
        # s.scrollPaddingBlockStart = None
        # s.scrollPaddingBottom = None
        # s.scrollPaddingInline = None
        # s.scrollPaddingInlineEnd = None
        # s.scrollPaddingInlineStart = None
        # s.scrollPaddingLeft = None
        # s.scrollPaddingRight = None
        # s.scrollPaddingTop = None
        # s.scrollSnapAlign = None
        # s.scrollSnapStop = None
        # s.scrollSnapType = None
        # s.shapeImageThreshold = None
        # s.shapeMargin = None
        # s.shapeOutside = None
        # s.shapeRendering = None
        # s.size = None
        # s.speak = None
        # s.src = None
        # s.stopColor = None
        # s.stopOpacity = None
        # s.stroke = None
        # s.strokeDasharray = None
        # s.strokeDashoffset = None
        # s.strokeLinecap = None
        # s.strokeLinejoin = None
        # s.strokeMiterlimit = None
        # s.strokeOpacity = None
        # s.strokeWidth = None
        # s.syntax = None
        # s.textAnchor = None
        # s.textCombineUpright = None
        # s.textDecorationSkipInk = None
        # s.textOrientation = None
        # s.textRendering = None
        # s.textSizeAdjust = None
        # s.textUnderlinePosition = None
        # s.touchAction = None
        # s.transformBox = None
        # s.unicodeRange = None
        # s.userZoom = None
        # s.vectorEffect = None
        # s.willChange = None
        # s.writingMode = None
        # s.x = None
        # s.y = None









if __name__ == '__main__':
    unittest.main()



"""
    domonic.constants.entities
    ====================================

"""


class Entity():

    # __slots__ = ('entity',)

    def __init__(self, entity: str) -> None:
        self.entity = entity

    def __str__(self) -> str:
        import html
        return html.unescape(self.character)


class Char():

    def __init__(self, character: str) -> None:
        self.character = character

    def __str__(self) -> str:
        import html
        return html.escape(self.character)

    # def __repr__(self):
    #     return self.character

    # web
    # ASCII Characters (Printable)
    SPACE: str = '&#32;'
    EXCLAMATION_MARK: str = '&#33;'  #: !
    QUOTATION_MARK: str = '&#34;'  #: "
    NUMBER_SIGN: str = '&#35;'  #: #
    DOLLAR_SIGN: str = '&#36;'  #: $
    PERCENT_SIGN: str = '&#37;'  #: %
    AMPERSAND: str = '&amp;'  #: &
    APOSTROPHE: str = '&#39;'  #: '
    OPENING_PARENTHESIS: str = '&#40;'  #: (
    LEFT_PARENTHESIS: str = '&#40;'  #: (
    CLOSING_PARENTHESIS: str = '&#41;'  #: )
    RIGHT_PARENTHESIS: str = '&#41;'  #: )
    ASTERISK: str = '&#42;'  #: *
    PLUS_SIGN: str = '&#43;'  #: +
    COMMA: str = '&#44;'  #: ,
    HYPHEN: str = '&#45;'  #: -
    PERIOD: str = '&#46;'  #: .
    SLASH: str = '&#47;'  #: /
    ZERO: str = '&#48;'  #: 0
    ONE: str = '&#49;'  #: 1
    TWO: str = '&#50;'  #: 2
    THREE: str = '&#51;'  #: 3
    FOUR: str = '&#52;'  #: 4
    FIVE: str = '&#53;'  #: 5
    SIX: str = '&#54;'  #: 6
    SEVEN: str = '&#55;'  #: 7
    EIGHT: str = '&#56;'  #: 8
    NINE: str = '&#57;'  #: 9

    COLON: str = '&#58;'  #: :
    SEMICOLON: str = '&#59;'  #: ;
    LESS_THAN: str = '&lt;'  #: <
    EQUALS_SIGN: str = '&#61;'  # :: str =
    GREATER_THAN: str = '&gt;'  #: >
    QUESTION_MARK: str = '&#63;'  #: ?
    AT_SIGN: str = '&#64;'  #: @

    UPPERCASE_A: str = '&#65;'  #: A
    UPPERCASE_B: str = '&#66;'  #: B
    UPPERCASE_C: str = '&#67;'  #: C
    UPPERCASE_D: str = '&#68;'  #: D
    UPPERCASE_E: str = '&#69;'  #: E
    UPPERCASE_F: str = '&#70;'  #: F
    UPPERCASE_G: str = '&#71;'  #: G
    UPPERCASE_H: str = '&#72;'  #: H
    UPPERCASE_I: str = '&#73;'  #: I
    UPPERCASE_J: str = '&#74;'  #: J
    UPPERCASE_K: str = '&#75;'  #: K
    UPPERCASE_L: str = '&#76;'  #: L
    UPPERCASE_M: str = '&#77;'  #: M
    UPPERCASE_N: str = '&#78;'  #: N
    UPPERCASE_O: str = '&#79;'  #: O
    UPPERCASE_P: str = '&#80;'  #: P
    UPPERCASE_Q: str = '&#81;'  #: Q
    UPPERCASE_R: str = '&#82;'  #: R
    UPPERCASE_S: str = '&#83;'  #: S
    UPPERCASE_T: str = '&#84;'  #: T
    UPPERCASE_U: str = '&#85;'  #: U
    UPPERCASE_V: str = '&#86;'  #: V
    UPPERCASE_W: str = '&#87;'  #: W
    UPPERCASE_X: str = '&#88;'  #: X
    UPPERCASE_Y: str = '&#89;'  #: Y
    UPPERCASE_Z: str = '&#90;'  #: Z

    OPENING_SQUARE_BRACKET: str = '&#91;'  #: [
    BACKSLASH: str = '&#92;'  #: \
    CLOSING_SQUARE_BRACKET: str = '&#93;'  #: ]
    CARET: str = '&#94;'  #: ^
    UNDERSCORE: str = '&#95;'  #: _
    GRAVE_ACCENT: str = '&#96;'  #:
    LOWERCASE_A: str = '&#97;'  #: a
    LOWERCASE_B: str = '&#98;'  #: b
    LOWERCASE_C: str = '&#99;'  #: c
    LOWERCASE_D: str = '&#100;'  #: d
    LOWERCASE_E: str = '&#101;'  #: e
    LOWERCASE_F: str = '&#102;'  #: f
    LOWERCASE_G: str = '&#103;'  #: g
    LOWERCASE_H: str = '&#104;'  #: h
    LOWERCASE_I: str = '&#105;'  #: i
    LOWERCASE_J: str = '&#106;'  #: j
    LOWERCASE_K: str = '&#107;'  #: k
    LOWERCASE_L: str = '&#108;'  #: l
    LOWERCASE_M: str = '&#109;'  #: m
    LOWERCASE_N: str = '&#110;'  #: n
    LOWERCASE_O: str = '&#111;'  #: o
    LOWERCASE_P: str = '&#112;'  #: p
    LOWERCASE_Q: str = '&#113;'  #: q
    LOWERCASE_R: str = '&#114;'  #: r
    LOWERCASE_S: str = '&#115;'  #: s
    LOWERCASE_T: str = '&#116;'  #: t
    LOWERCASE_U: str = '&#117;'  #: u
    LOWERCASE_V: str = '&#118;'  #: v
    LOWERCASE_W: str = '&#119;'  #: w
    LOWERCASE_X: str = '&#120;'  #: x
    LOWERCASE_Y: str = '&#121;'  #: y
    LOWERCASE_Z: str = '&#122;'  #: z

    OPENING_CURLY_BRACE: str = '&#123;'  #: {
    LEFT_CURLY_BRACE: str = '&#123;'  #: {
    VERTICAL_BAR: str = '&#124;'  #: |
    CLOSING_CURLY_BRACE: str = '&#125;'  #: }
    RIGHT_CURLY_BRACE: str = '&#125;'  #: }
    TILDE: str = '&#126;'  #: ~

    # ISO-8859-1 Characters
    AGRAVE: str = '&Agrave;'  #: À
    AACUTE: str = '&Aacute;'  #: Á
    ACIRC: str = '&Acirc;'  #: Â
    ATILDE: str = '&Atilde;'  #: Ã
    AUML: str = '&Auml;'  #: Ä
    ARING: str = '&Aring;'  #: Å
    AELIG: str = '&AElig;'  #: Æ
    CCEDIL: str = '&Ccedil;'  #: Ç
    EGRAVE: str = '&Egrave;'  #: È
    EACUTE: str = '&Eacute;'  #: É
    ECIRC: str = '&Ecirc;'  #: Ê
    EUML: str = '&Euml;'  #: Ë
    IGRAVE: str = '&Igrave;'  #: Ì
    IACUTE: str = '&Iacute;'  #: Í
    ICIRC: str = '&Icirc;'  #: Î
    IUML: str = '&Iuml;'  #: Ï
    ETH: str = '&ETH;'  #: Ð
    NTILDE: str = '&Ntilde;'  #: Ñ
    OGRAVE: str = '&Ograve;'  #: Ò
    OACUTE: str = '&Oacute;'  #: Ó
    OCIRC: str = '&Ocirc;'  #: Ô
    OTILDE: str = '&Otilde;'  #: Õ
    OUML: str = '&Ouml;'  #: Ö
    OSLASH: str = '&Oslash;'  #: Ø
    UGRAVE: str = '&Ugrave;'  #: Ù
    UACUTE: str = '&Uacute;'  #: Ú
    UCIRC: str = '&Ucirc;'  #: Û
    UUML: str = '&Uuml;'  #: Ü
    YACUTE: str = '&Yacute;'  #: Ý
    THORN: str = '&THORN;'  #: Þ
    SZLIG: str = '&szlig;'  #: ß
    AGRAVE: str = '&agrave;'  #: à
    AACUTE: str = '&aacute;'  #: á
    ACIRC: str = '&acirc;'  #: â
    ATILDE: str = '&atilde;'  #: ã
    AUML: str = '&auml;'  #: ä
    ARING: str = '&aring;'  #: å
    AELIG: str = '&aelig;'  #: æ
    CCEDIL: str = '&ccedil;'  #: ç
    EGRAVE: str = '&egrave;'  #: è
    EACUTE: str = '&eacute;'  #: é
    ECIRC: str = '&ecirc;'  #: ê
    EUML: str = '&euml;'  #: ë
    IGRAVE: str = '&igrave;'  #: ì
    IACUTE: str = '&iacute;'  #: í
    ICIRC: str = '&icirc;'  #: î
    IUML: str = '&iuml;'  #: ï
    ETH: str = '&eth;'  #: ð
    NTILDE: str = '&ntilde;'  #: ñ
    OGRAVE: str = '&ograve;'  #: ò
    OACUTE: str = '&oacute;'  #: ó
    OCIRC: str = '&ocirc;'  #: ô
    OTILDE: str = '&otilde;'  #: õ
    OUML: str = '&ouml;'  #: ö
    OSLASH: str = '&oslash;'  #: ø
    UGRAVE: str = '&ugrave;'  #: ù
    UACUTE: str = '&uacute;'  #: ú
    UCIRC: str = '&ucirc;'  #: û
    UUML: str = '&uuml;'  #: ü
    YACUTE: str = '&yacute;'  #: ý
    THORN: str = '&thorn;'  #: þ
    YUML: str = '&yuml;'  #: ÿ

    # ISO-8859-1 Symbols
    NBSP: str = '&nbsp;'  #:
    IEXCL: str = '&iexcl;'  #: ¡
    CENT: str = '&cent;'  #: ¢
    POUND: str = '&pound;'  #: £
    CURREN: str = '&curren;'  #: ¤
    YEN: str = '&yen;'  #: ¥
    BRVBAR: str = '&brvbar;'  #: ¦
    SECT: str = '&sect;'  #: §
    UML: str = '&uml;'  #: ¨
    COPY: str = '&copy;'  #: ©
    COPYRIGHT: str = '&copy;'  #: ©
    ORDF: str = '&ordf;'  #: ª
    LAQUO: str = '&laquo;'  #: «
    NOT: str = '&not;'  #: ¬
    # ­   &shy;   &#173;  Soft hyphen
    REG: str = '&reg;'  #: ®
    MACR: str = '&macr;'  #: ¯
    DEG: str = '&deg;'  #: °
    PLUSMN: str = '&plusmn;'  #: ±
    SUP2: str = '&sup2;'  #: ²
    SUP3: str = '&sup3;'  #: ³
    ACUTE: str = '&acute;'  #: ´
    MICRO: str = '&micro;'  #: µ
    PARA: str = '&para;'  #: ¶
    CEDIL: str = '&cedil;'  #: ¸
    SUP1: str = '&sup1;'  #: ¹
    ORDM: str = '&ordm;'  #: º
    RAQUO: str = '&raquo;'  #: »
    FRAC14: str = '&frac14;'  #: ¼
    FRAC12: str = '&frac12;'  #: ½
    FRAC34: str = '&frac34;'  #: ¾
    IQUEST: str = '&iquest;'  #: ¿
    TIMES: str = '&times;'  #: ×
    DIVIDE: str = '&divide;'  #: ÷

    # Math Symbols
    FORALL: str = '&forall;'  #: ∀
    PART: str = '&part;'  #: ∂
    EXIST: str = '&exist;'  #: ∃
    EMPTY: str = '&empty;'  #: ∅
    NABLA: str = '&nabla;'  #: ∇
    ISIN: str = '&isin;'  #: ∈
    NOTIN: str = '&notin;'  #: ∉
    NI: str = '&ni;'  #: ∋
    PROD: str = '&prod;'  #: ∏
    SUM: str = '&sum;'  #: ∑
    MINUS: str = '&minus;'  #: −
    LOWAST: str = '&lowast;'  #: ∗
    RADIC: str = '&radic;'  #: √
    PROP: str = '&prop;'  #: ∝
    INFIN: str = '&infin;'  #: ∞
    ANG: str = '&ang;'  #: ∠
    AND: str = '&and;'  #: ∧
    OR: str = '&or;'  #: ∨
    CAP: str = '&cap;'  #: ∩
    CUP: str = '&cup;'  #: ∪
    INT: str = '&int;'  #: ∫
    THERE4: str = '&there4;'  #: ∴
    SIM: str = '&sim;'  #: ∼
    CONG: str = '&cong;'  #: ≅
    ASYMP: str = '&asymp;'  #: ≈
    NE: str = '&ne;'  #: ≠
    EQUIV: str = '&equiv;'  #: ≡
    LE: str = '&le;'  #: ≤
    GE: str = '&ge;'  #: ≥
    SUB: str = '&sub;'  #: ⊂
    SUP: str = '&sup;'  #: ⊃
    NSUB: str = '&nsub;'  #: ⊄
    SUBE: str = '&sube;'  #: ⊆
    SUPE: str = '&supe;'  #: ⊇
    OPLUS: str = '&oplus;'  #: ⊕
    OTIMES: str = '&otimes;'  #: ⊗
    PERP: str = '&perp;'  #: ⊥
    SDOT: str = '&sdot;'  #: ⋅

    # Greek Letters
    ALPHA: str = '&Alpha;'  #: Α
    BETA: str = '&Beta;'  #: Β
    GAMMA: str = '&Gamma;'  #: Γ
    DELTA: str = '&Delta;'  #: Δ
    EPSILON: str = '&Epsilon;'  #: Ε
    ZETA: str = '&Zeta;'  #: Ζ
    ETA: str = '&Eta;'  #: Η
    THETA: str = '&Theta;'  #: Θ
    IOTA: str = '&Iota;'  #: Ι
    KAPPA: str = '&Kappa;'  #: Κ
    LAMBDA: str = '&Lambda;'  #: Λ
    MU: str = '&Mu;'  #: Μ
    NU: str = '&Nu;'  #: Ν
    XI: str = '&Xi;'  #: Ξ
    OMICRON: str = '&Omicron;'  #: Ο
    PI: str = '&Pi;'  #: Π
    RHO: str = '&Rho;'  #: Ρ
    SIGMA: str = '&Sigma;'  #: Σ
    TAU: str = '&Tau;'  #: Τ
    UPSILON: str = '&Upsilon;'  #: Υ
    PHI: str = '&Phi;'  #: Φ
    CHI: str = '&Chi;'  #: Χ
    PSI: str = '&Psi;'  #: Ψ
    OMEGA: str = '&Omega;'  #: Ω
    ALPHA: str = '&alpha;'  #: α
    BETA: str = '&beta;'  #: β
    GAMMA: str = '&gamma;'  #: γ
    DELTA: str = '&delta;'  #: δ
    EPSILON: str = '&epsilon;'  #: ε
    ZETA: str = '&zeta;'  #: ζ
    ETA: str = '&eta;'  #: η
    THETA: str = '&theta;'  #: θ
    IOTA: str = '&iota;'  #: ι
    KAPPA: str = '&kappa;'  #: κ
    LAMBDA: str = '&lambda;'  #: λ
    MU: str = '&mu;'  #: μ
    NU: str = '&nu;'  #: ν
    XI: str = '&xi;'  #: ξ
    OMICRON: str = '&omicron;'  #: ο
    PI: str = '&pi;'  #: π
    RHO: str = '&rho;'  #: ρ
    SIGMAF: str = '&sigmaf;'  #: ς
    SIGMA: str = '&sigma;'  #: σ
    TAU: str = '&tau;'  #: τ
    UPSILON: str = '&upsilon;'  #: υ
    PHI: str = '&phi;'  #: φ
    CHI: str = '&chi;'  #: χ
    PSI: str = '&psi;'  #: ψ
    OMEGA: str = '&omega;'  #: ω
    THETASYM: str = '&thetasym;'  #: ϑ
    UPSIH: str = '&upsih;'  #: ϒ
    PIV: str = '&piv;'  #: ϖ

    OELIG: str = '&OElig;'  #: Œ
    oeLIG: str = '&oelig;'  #: œ
    SCARON: str = '&Scaron;'  #: Š
    Scaron: str = '&Scaron;'  #: Š
    scaron: str = '&scaron;'  #: š
    YUML: str = '&Yuml;'  #: Ÿ
    FNOF: str = '&fnof;'  #: ƒ
    CIRC: str = '&circ;'  #: ˆ
    TILDE: str = '&tilde;'  #: ˜

    #     &ensp;  &#8194; En space
    #     &emsp;  &#8195; Em space
    #     &thinsp;    &#8201; Thin space
    # ‌   &zwnj;  &#8204; Zero width non-joiner
    # ‍   &zwj;   &#8205; Zero width joiner
    # ‎   &lrm;   &#8206; Left-to-right mark
    # ‏   &rlm;   &#8207; Right-to-left mark

    NDASH: str = '&ndash;'  #: –
    MDASH: str = '&mdash;'  #: —
    LSQUO: str = '&lsquo;'  #: ‘
    RSQUO: str = '&rsquo;'  #: ’
    SBQUO: str = '&sbquo;'  #: ‚
    LDQUO: str = '&ldquo;'  #: “
    RDQUO: str = '&rdquo;'  #: ”
    BDQUO: str = '&bdquo;'  #: „
    DAGGER: str = '&dagger;'  #: †
    DAGGER: str = '&Dagger;'  #: ‡
    BULL: str = '&bull;'  #: •
    HELLIP: str = '&hellip;'  #: …
    PERMIL: str = '&permil;'  #: ‰
    PRIME: str = '&prime;'  #: ′
    PRIME: str = '&Prime;'  #: ″
    LSAQUO: str = '&lsaquo;'  #: ‹
    RSAQUO: str = '&rsaquo;'  #: ›
    OLINE: str = '&oline;'  #: ‾
    EURO: str = '&euro;'  #: €
    TRADE: str = '&trade;'  #: ™
    TRADEMARK: str = '&trade;'  #: ™

    # ARROWS
    LARR: str = '&larr;'  #: ←
    LEFT: str = '&larr;'  #: ←
    UARR: str = '&uarr;'  #: ↑
    UP: str = '&uarr;'  #: ↑
    RARR: str = '&rarr;'  #: →
    RIGHT: str = '&rarr;'  #: →
    DARR: str = '&darr;'  #: ↓
    DOWN: str = '&darr;'  #: ↓

    HARR: str = '&harr;'  #: ↔
    CRARR: str = '&crarr;'  #: ↵
    LCEIL: str = '&lceil;'  #: ⌈
    RCEIL: str = '&rceil;'  #: ⌉
    LFLOOR: str = '&lfloor;'  #: ⌊
    RFLOOR: str = '&rfloor;'  #: ⌋
    LOZ: str = '&loz;'  #: ◊

    SPADES: str = '&spades;'  #: ♠
    CLUBS: str = '&clubs;'  #: ♣
    HEARTS: str = '&hearts;'  #: ♥
    DIAMS: str = '&diams;'  #: ♦
    DIAMONDS: str = '&diams;'  #: ♦

    SUNG: str = '&sung;'  #: ♪
    FLAT: str = '&flat;'  #: ♭
    NATUR: str = '&natur;'  #: ♮
    NATURAL: str = '&natural;'  #: ♮
    SHARP: str = '&sharp;'  #: ♯

    CHECK: str = "&check;"  #: ✓
    CHECKMARK: str = "&checkmark;"  #: ✓
    TICK: str = "&check;"  #: ✓
    CROSS: str = "&cross;"  #: ✗

    OHM: str = '&ohm;'  #: Ω
    MHO: str = '&mho;'  #: ℧

    FRAC13: str = '&frac13;'  #: ⅓
    FRAC23: str = '&frac23;'  #: ⅔
    FRAC15: str = '&frac15;'  #: ⅕
    FRAC25: str = '&frac25;'  #: ⅖
    FRAC35: str = '&frac35;'  #: ⅗
    FRAC45: str = '&frac45;'  #: ⅘
    FRAC16: str = '&frac16;'  #: ⅙
    FRAC56: str = '&frac56;'  #: ⅚
    FRAC18: str = '&frac18;'  #: ⅛
    FRAC38: str = '&frac38;'  #: ⅜
    FRAC58: str = '&frac58;'  #: ⅝
    FRAC78: str = '&frac78;'  #: ⅞

    STAR: str = "&star;"  #: ☆
    STARF: str = "&starf;"  #: ★
    BIGSTAR: str = "&bigstar;"
    PHONE: str = "&phone;"  #: ☎
    FEMALE: str = "&female;"  #: ♀
    MALE: str = "&male;"  #: ♂



"""
    domonic.collada
    ====================================
    Generate Collada with python 3

"""

from domonic.dom import Element
from domonic.html import closed_tag


def write(inp, outp="output"):
    """write

    Renders the input to string or .dae file

    Args:
        inp (obj): domonic.collada tags
        outp (str): An optional filename for the .dae file that gets produced

    Returns:
        str: A HTML rendered string
    """
    if outp != "":
        with open(outp, "w+") as f:
            f.write(str(inp) + ".dae")
    return str(inp)


COLLADA = type("COLLADA", (Element,), {"name": "COLLADA"})

asset = type("asset", (Element,), {"name": "asset"})
library_cameras = type("library_cameras", (Element,), {"name": "library_cameras"})
library_lights = type("library_lights", (Element,), {"name": "library_lights"})
library_materials = type("library_materials", (Element,), {"name": "library_materials"})
library_effects = type("library_effects", (Element,), {"name": "library_effects"})
library_geometries = type(
    "library_geometries", (Element,), {"name": "library_geometries"}
)
library_visual_scenes = type(
    "library_visual_scenes", (Element,), {"name": "library_visual_scenes"}
)
scene = type("scene", (Element,), {"name": "scene"})
extra = type("extra", (Element,), {"name": "extra"})
author = type("author", (Element,), {"name": "author"})
up_axis = type("up_axis", (Element,), {"name": "up_axis"})
camera = type("camera", (Element,), {"name": "camera"})
optics = type("optics", (Element,), {"name": "optics"})
technique = type("technique", (Element,), {"name": "technique"})
technique_common = type("technique_common", (Element,), {"name": "technique_common"})
aspect_ratio = type("aspect_ratio", (Element,), {"name": "aspect_ratio"})
light = type("light", (Element,), {"name": "light"})
constant_attenuation = type(
    "constant_attenuation", (Element,), {"name": "constant_attenuation"}
)
linear_attenuation = type(
    "linear_attenuation", (Element,), {"name": "linear_attenuation"}
)
quadratic_attenuation = type(
    "quadratic_attenuation", (Element,), {"name": "quadratic_attenuation"}
)
material = type("material", (Element,), {"name": "material"})
effect = type("effect", (Element,), {"name": "effect"})
profile_COMMON = type("profile_COMMON", (Element,), {"name": "profile_COMMON"})
phong = type("phong", (Element,), {"name": "phong"})
emission = type("emission", (Element,), {"name": "emission"})
geometry = type("geometry", (Element,), {"name": "geometry"})
mesh = type("mesh", (Element,), {"name": "mesh"})
source = type("source", (Element,), {"name": "source"})
vertices = type("vertices", (Element,), {"name": "vertices"})
polylist = type("polylist", (Element,), {"name": "polylist"})
vcount = type("vcount", (Element,), {"name": "vcount"})
p = type("p", (Element,), {"name": "p"})
library_visual_scenes = type(
    "library_visual_scenes", (Element,), {"name": "library_visual_scenes"}
)
visual_scene = type("visual_scene", (Element,), {"name": "visual_scene"})
node = type("node", (Element,), {"name": "node"})
instance_visual_scene = type(
    "instance_visual_scene", (Element,), {"name": "instance_visual_scene"}
)
test_element = type("test_element", (Element,), {"name": "test_element"})
instance_effect = type("instance_effect", (closed_tag,), {"name": "instance_effect"})
texture_unit = type("texture_unit", (closed_tag,), {"name": "texture_unit"})
tapered_cylinder = type("tapered_cylinder", (closed_tag,), {"name": "tapered_cylinder"})
tapered_capsule = type("tapered_capsule", (closed_tag,), {"name": "tapered_capsule"})
format_hint = type("format_hint", (closed_tag,), {"name": "format_hint"})
revisions = type("revisions", (closed_tag,), {"name": "revisions"})
contributer = type("contributer", (closed_tag,), {"name": "contributer"})
acceleration = type("acceleration", (Element,), {"name": "acceleration"})
axis_info = type("axis_info", (Element,), {"name": "axis_info"})
accessor = type("accessor", (Element,), {"name": "accessor"})
active = type("active", (Element,), {"name": "active"})
alpha_func = type("alpha_func", (Element,), {"name": "alpha_func"})
alpha_test_enable = type("alpha_test_enable", (Element,), {"name": "alpha_test_enable"})
alpha = type("alpha", (Element,), {"name": "alpha"})
altitude = type("altitude", (Element,), {"name": "altitude"})
ambient = type("ambient", (Element,), {"name": "ambient"})
angle = type("angle", (Element,), {"name": "angle"})
angular_velocity = type("angular_velocity", (Element,), {"name": "angular_velocity"})
angular = type("angular", (Element,), {"name": "angular"})
animation_clip = type("animation_clip", (Element,), {"name": "animation_clip"})
animation = type("animation", (Element,), {"name": "animation"})
annotate = type("annotate", (Element,), {"name": "annotate"})
argument = type("argument", (Element,), {"name": "argument"})
array = type("array", (Element,), {"name": "array"})
articulated_system = type(
    "articulated_system", (Element,), {"name": "articulated_system"}
)
perspective = type("perspective", (Element,), {"name": "perspective"})
asset = type("asset", (Element,), {"name": "asset"})
se = type("se", (Element,), {"name": "se"})
attachment_end = type("attachment_end", (Element,), {"name": "attachment_end"})
attachment_full = type("attachment_full", (Element,), {"name": "attachment_full"})
attachment_start = type("attachment_start", (Element,), {"name": "attachment_start"})
attachment = type("attachment", (Element,), {"name": "attachment"})
author_email = type("author_email", (Element,), {"name": "author_email"})
author_website = type("author_website", (Element,), {"name": "author_website"})
author = type("author", (Element,), {"name": "author"})
authoring_tool = type("authoring_tool", (Element,), {"name": "authoring_tool"})
auto_normal_enable = type(
    "auto_normal_enable", (Element,), {"name": "auto_normal_enable"}
)
axis_info = type("axis_info", (Element,), {"name": "axis_info"})
axis = type("axis", (Element,), {"name": "axis"})
prismatic = type("prismatic", (Element,), {"name": "prismatic"})
revolute = type("revolute", (Element,), {"name": "revolute"})
swept_surface = type("swept_surface", (Element,), {"name": "swept_surface"})
binary = type("binary", (Element,), {"name": "binary"})
bind_attribute = type("bind_attribute", (Element,), {"name": "bind_attribute"})
bind_joint_axis = type("bind_joint_axis", (Element,), {"name": "bind_joint_axis"})
bind_kinematics_model = type(
    "bind_kinematics_model", (Element,), {"name": "bind_kinematics_model"}
)
bind_shape_matrix = type("bind_shape_matrix", (Element,), {"name": "bind_shape_matrix"})
bind_uniform = type("bind_uniform", (Element,), {"name": "bind_uniform"})
bind_vertex_input = type("bind_vertex_input", (Element,), {"name": "bind_vertex_input"})
bind = type("bind", (Element,), {"name": "bind"})
bind = type("bind", (Element,), {"name": "bind"})
blend_color = type("blend_color", (Element,), {"name": "blend_color"})
blend_enable = type("blend_enable", (Element,), {"name": "blend_enable"})
blend_equation_separate = type(
    "blend_equation_separate", (Element,), {"name": "blend_equation_separate"}
)
blend_equation = type("blend_equation", (Element,), {"name": "blend_equation"})
blend_func_separate = type(
    "blend_func_separate", (Element,), {"name": "blend_func_separate"}
)
blend_func = type("blend_func", (Element,), {"name": "blend_func"})
blinn = type("blinn", (Element,), {"name": "blinn"})
bool_array = type("bool_array", (Element,), {"name": "bool_array"})
border_color = type("border_color", (Element,), {"name": "border_color"})
box = type("box", (Element,), {"name": "box"})
brep = type("brep", (Element,), {"name": "brep"})
camera = type("camera", (Element,), {"name": "camera"})
capsule = type("capsule", (Element,), {"name": "capsule"})
channel = type("channel", (Element,), {"name": "channel"})
arget = type("arget", (Element,), {"name": "arget"})
circle = type("circle", (Element,), {"name": "circle"})
clip_plane_enable = type("clip_plane_enable", (Element,), {"name": "clip_plane_enable"})
clip_plane = type("clip_plane", (Element,), {"name": "clip_plane"})
code = type("code", (Element,), {"name": "code"})
color_clear = type("color_clear", (Element,), {"name": "color_clear"})
color_logic_op_enable = type(
    "color_logic_op_enable", (Element,), {"name": "color_logic_op_enable"}
)
color_mask = type("color_mask", (Element,), {"name": "color_mask"})
color_material_enable = type(
    "color_material_enable", (Element,), {"name": "color_material_enable"}
)
color_material = type("color_material", (Element,), {"name": "color_material"})
color_target = type("color_target", (Element,), {"name": "color_target"})
color = type("color", (Element,), {"name": "color"})
comments = type("comments", (Element,), {"name": "comments"})
compiler = type("compiler", (Element,), {"name": "compiler"})
cone = type("cone", (Element,), {"name": "cone"})
connect_param = type("connect_param", (Element,), {"name": "connect_param"})
point = type("point", (Element,), {"name": "point"})
spot = type("spot", (Element,), {"name": "spot"})
constant = type("constant", (Element,), {"name": "constant"})
constant = type("constant", (Element,), {"name": "constant"})
contributor = type("contributor", (Element,), {"name": "contributor"})
control_vertices = type("control_vertices", (Element,), {"name": "control_vertices"})
controller = type("controller", (Element,), {"name": "controller"})
convex_mesh = type("convex_mesh", (Element,), {"name": "convex_mesh"})
copyright = type("copyright", (Element,), {"name": "copyright"})
coverage = type("coverage", (Element,), {"name": "coverage"})
create_2d = type("create_2d", (Element,), {"name": "create_2d"})
create_3d = type("create_3d", (Element,), {"name": "create_3d"})
create_cube = type("create_cube", (Element,), {"name": "create_cube"})
created = type("created", (Element,), {"name": "created"})
cull_face_enable = type("cull_face_enable", (Element,), {"name": "cull_face_enable"})
cull_face = type("cull_face", (Element,), {"name": "cull_face"})
curve = type("curve", (Element,), {"name": "curve"})
curves = type("curves", (Element,), {"name": "curves"})
cylinder = type("cylinder", (Element,), {"name": "cylinder"})
cylinder = type("cylinder", (Element,), {"name": "cylinder"})
damping = type("damping", (Element,), {"name": "damping"})
deceleration = type("deceleration", (Element,), {"name": "deceleration"})
axis_info = type("axis_info", (Element,), {"name": "axis_info"})
effector_info = type("effector_info", (Element,), {"name": "effector_info"})
density = type("density", (Element,), {"name": "density"})
depth_bounds_enable = type(
    "depth_bounds_enable", (Element,), {"name": "depth_bounds_enable"}
)
depth_bounds = type("depth_bounds", (Element,), {"name": "depth_bounds"})
depth_clamp_enable = type(
    "depth_clamp_enable", (Element,), {"name": "depth_clamp_enable"}
)
depth_clear = type("depth_clear", (Element,), {"name": "depth_clear"})
depth_func = type("depth_func", (Element,), {"name": "depth_func"})
depth_mask = type("depth_mask", (Element,), {"name": "depth_mask"})
depth_range = type("depth_range", (Element,), {"name": "depth_range"})
depth_target = type("depth_target", (Element,), {"name": "depth_target"})
depth_test_enable = type("depth_test_enable", (Element,), {"name": "depth_test_enable"})
diffuse = type("diffuse", (Element,), {"name": "diffuse"})
direction = type("direction", (Element,), {"name": "direction"})
line = type("line", (Element,), {"name": "line"})
swept_surface = type("swept_surface", (Element,), {"name": "swept_surface"})
directional = type("directional", (Element,), {"name": "directional"})
dither_enable = type("dither_enable", (Element,), {"name": "dither_enable"})
draw = type("draw", (Element,), {"name": "draw"})
dynamic_friction = type("dynamic_friction", (Element,), {"name": "dynamic_friction"})
dynamic = type("dynamic", (Element,), {"name": "dynamic"})
edges = type("edges", (Element,), {"name": "edges"})
effect = type("effect", (Element,), {"name": "effect"})
effector_info = type("effector_info", (Element,), {"name": "effector_info"})
ellipse = type("ellipse", (Element,), {"name": "ellipse"})
emission = type("emission", (Element,), {"name": "emission"})
enabled = type("enabled", (Element,), {"name": "enabled"})
equation = type("equation", (Element,), {"name": "equation"})
evaluate_scene = type("evaluate_scene", (Element,), {"name": "evaluate_scene"})
evaluate = type("evaluate", (Element,), {"name": "evaluate"})
exact = type("exact", (Element,), {"name": "exact"})
extra = type("extra", (Element,), {"name": "extra"})
faces = type("faces", (Element,), {"name": "faces"})
falloff_angle = type("falloff_angle", (Element,), {"name": "falloff_angle"})
falloff_exponent = type("falloff_exponent", (Element,), {"name": "falloff_exponent"})
float_array = type("float_array", (Element,), {"name": "float_array"})
focal = type("focal", (Element,), {"name": "focal"})
fog_color = type("fog_color", (Element,), {"name": "fog_color"})
fog_coord_src = type("fog_coord_src", (Element,), {"name": "fog_coord_src"})
fog_density = type("fog_density", (Element,), {"name": "fog_density"})
fog_enable = type("fog_enable", (Element,), {"name": "fog_enable"})
fog_end = type("fog_end", (Element,), {"name": "fog_end"})
fog_mode = type("fog_mode", (Element,), {"name": "fog_mode"})
fog_state = type("fog_state", (Element,), {"name": "fog_state"})
force_field = type("force_field", (Element,), {"name": "force_field"})
formula = type("formula", (Element,), {"name": "formula"})
frame_object = type("frame_object", (Element,), {"name": "frame_object"})
frame_origin = type("frame_origin", (Element,), {"name": "frame_origin"})
frame_tcp = type("frame_tcp", (Element,), {"name": "frame_tcp"})
frame_tip = type("frame_tip", (Element,), {"name": "frame_tip"})
front_face = type("front_face", (Element,), {"name": "front_face"})
func = type("func", (Element,), {"name": "func"})
geographic_location = type(
    "geographic_location", (Element,), {"name": "geographic_location"}
)
geometry = type("geometry", (Element,), {"name": "geometry"})
gravity = type("gravity", (Element,), {"name": "gravity"})
h = type("h", (Element,), {"name": "h"})
half_extents = type("half_extents", (Element,), {"name": "half_extents"})
height = type("height", (Element,), {"name": "height"})
capsule = type("capsule", (Element,), {"name": "capsule"})
binary = type("binary", (Element,), {"name": "binary"})
init_from = type("init_from", (Element,), {"name": "init_from"})
hint = type("hint", (Element,), {"name": "hint"})
hollow = type("hollow", (Element,), {"name": "hollow"})
IDREF_array = type("IDREF_array", (Element,), {"name": "IDREF_array"})
image = type("image", (Element,), {"name": "image"})
imager = type("imager", (Element,), {"name": "imager"})
include = type("include", (Element,), {"name": "include"})
index_of_refraction = type(
    "index_of_refraction", (Element,), {"name": "index_of_refraction"}
)
index = type("index", (Element,), {"name": "index"})
inertia = type("inertia", (Element,), {"name": "inertia"})
instance_rigid_body = type(
    "instance_rigid_body", (Element,), {"name": "instance_rigid_body"}
)
init_from = type("init_from", (Element,), {"name": "init_from"})
inline = type("inline", (Element,), {"name": "inline"})
emantic = type("emantic", (Element,), {"name": "emantic"})
emantics = type("emantics", (Element,), {"name": "emantics"})
instance_animation = type(
    "instance_animation", (Element,), {"name": "instance_animation"}
)
instance_articulated_system = type(
    "instance_articulated_system", (Element,), {"name": "instance_articulated_system"}
)
instance_camera = type("instance_camera", (Element,), {"name": "instance_camera"})
instance_controller = type(
    "instance_controller", (Element,), {"name": "instance_controller"}
)
instance_effect = type("instance_effect", (Element,), {"name": "instance_effect"})
instance_force_field = type(
    "instance_force_field", (Element,), {"name": "instance_force_field"}
)
instance_formula = type("instance_formula", (Element,), {"name": "instance_formula"})
instance_geometry = type("instance_geometry", (Element,), {"name": "instance_geometry"})
instance_image = type("instance_image", (Element,), {"name": "instance_image"})
instance_joint = type("instance_joint", (Element,), {"name": "instance_joint"})
instance_kinematics_model = type(
    "instance_kinematics_model", (Element,), {"name": "instance_kinematics_model"}
)
instance_kinematics_scene = type(
    "instance_kinematics_scene", (Element,), {"name": "instance_kinematics_scene"}
)
instance_light = type("instance_light", (Element,), {"name": "instance_light"})
instance_material = type("instance_material", (Element,), {"name": "instance_material"})
instance_material = type("instance_material", (Element,), {"name": "instance_material"})
instance_node = type("instance_node", (Element,), {"name": "instance_node"})
instance_physics_material = type(
    "instance_physics_material", (Element,), {"name": "instance_physics_material"}
)
instance_physics_model = type(
    "instance_physics_model", (Element,), {"name": "instance_physics_model"}
)
instance_physics_scene = type(
    "instance_physics_scene", (Element,), {"name": "instance_physics_scene"}
)
instance_rigid_body = type(
    "instance_rigid_body", (Element,), {"name": "instance_rigid_body"}
)
instance_rigid_constraint = type(
    "instance_rigid_constraint", (Element,), {"name": "instance_rigid_constraint"}
)
instance_visual_scene = type(
    "instance_visual_scene", (Element,), {"name": "instance_visual_scene"}
)
int_array = type("int_array", (Element,), {"name": "int_array"})
interpenetrate = type("interpenetrate", (Element,), {"name": "interpenetrate"})
jerk = type("jerk", (Element,), {"name": "jerk"})
axis_info = type("axis_info", (Element,), {"name": "axis_info"})
joint = type("joint", (Element,), {"name": "joint"})
joints = type("joints", (Element,), {"name": "joints"})
keywords = type("keywords", (Element,), {"name": "keywords"})
kinematics_model = type("kinematics_model", (Element,), {"name": "kinematics_model"})
kinematics_scene = type("kinematics_scene", (Element,), {"name": "kinematics_scene"})
kinematics = type("kinematics", (Element,), {"name": "kinematics"})
lambert = type("lambert", (Element,), {"name": "lambert"})
latitude = type("latitude", (Element,), {"name": "latitude"})
layer = type("layer", (Element,), {"name": "layer"})
library_animation_clips = type(
    "library_animation_clips", (Element,), {"name": "library_animation_clips"}
)
library_animations = type(
    "library_animations", (Element,), {"name": "library_animations"}
)
library_articulated_systems = type(
    "library_articulated_systems", (Element,), {"name": "library_articulated_systems"}
)
library_cameras = type("library_cameras", (Element,), {"name": "library_cameras"})
library_controllers = type(
    "library_controllers", (Element,), {"name": "library_controllers"}
)
library_effects = type("library_effects", (Element,), {"name": "library_effects"})
library_force_fields = type(
    "library_force_fields", (Element,), {"name": "library_force_fields"}
)
library_formulas = type("library_formulas", (Element,), {"name": "library_formulas"})
library_geometries = type(
    "library_geometries", (Element,), {"name": "library_geometries"}
)
library_images = type("library_images", (Element,), {"name": "library_images"})
library_joints = type("library_joints", (Element,), {"name": "library_joints"})
library_kinematics_models = type(
    "library_kinematics_models", (Element,), {"name": "library_kinematics_models"}
)
library_kinematics_scenes = type(
    "library_kinematics_scenes", (Element,), {"name": "library_kinematics_scenes"}
)
library_nodes = type("library_nodes", (Element,), {"name": "library_nodes"})
library_physics_materials = type(
    "library_physics_materials", (Element,), {"name": "library_physics_materials"}
)
library_physics_models = type(
    "library_physics_models", (Element,), {"name": "library_physics_models"}
)
library_physics_scenes = type(
    "library_physics_scenes", (Element,), {"name": "library_physics_scenes"}
)
library_visual_scenes = type(
    "library_visual_scenes", (Element,), {"name": "library_visual_scenes"}
)
light_ambient = type("light_ambient", (Element,), {"name": "light_ambient"})
light_constant_attenuation = type(
    "light_constant_attenuation", (Element,), {"name": "light_constant_attenuation"}
)
light_diffuse = type("light_diffuse", (Element,), {"name": "light_diffuse"})
light_enable = type("light_enable", (Element,), {"name": "light_enable"})
light_linear_attenuation = type(
    "light_linear_attenuation", (Element,), {"name": "light_linear_attenuation"}
)
light_model_ambient = type(
    "light_model_ambient", (Element,), {"name": "light_model_ambient"}
)
light_model_color_control = type(
    "light_model_color_control", (Element,), {"name": "light_model_color_control"}
)
light_model_local_viewer_enable = type(
    "light_model_local_viewer_enable",
    (Element,),
    {"name": "light_model_local_viewer_enable"},
)
light_model_two_side_enable = type(
    "light_model_two_side_enable", (Element,), {"name": "light_model_two_side_enable"}
)
light_position = type("light_position", (Element,), {"name": "light_position"})
light_quadratic_attenuation = type(
    "light_quadratic_attenuation", (Element,), {"name": "light_quadratic_attenuation"}
)
light_specular = type("light_specular", (Element,), {"name": "light_specular"})
light_spot_cutoff = type("light_spot_cutoff", (Element,), {"name": "light_spot_cutoff"})
light_spot_direction = type(
    "light_spot_direction", (Element,), {"name": "light_spot_direction"}
)
light_spot_exponent = type(
    "light_spot_exponent", (Element,), {"name": "light_spot_exponent"}
)
lighting_enable = type("lighting_enable", (Element,), {"name": "lighting_enable"})
lights = type("lights", (Element,), {"name": "lights"})
limits = type("limits", (Element,), {"name": "limits"})
axis_info = type("axis_info", (Element,), {"name": "axis_info"})
prismatic = type("prismatic", (Element,), {"name": "prismatic"})
revolute = type("revolute", (Element,), {"name": "revolute"})
line_smooth_enable = type(
    "line_smooth_enable", (Element,), {"name": "line_smooth_enable"}
)
line_stipple_enable = type(
    "line_stipple_enable", (Element,), {"name": "line_stipple_enable"}
)
line_stipple = type("line_stipple", (Element,), {"name": "line_stipple"})
line_width = type("line_width", (Element,), {"name": "line_width"})
line = type("line", (Element,), {"name": "line"})
linear_attenuation = type(
    "linear_attenuation", (Element,), {"name": "linear_attenuation"}
)
spot = type("spot", (Element,), {"name": "spot"})
linear = type("linear", (Element,), {"name": "linear"})
lines = type("lines", (Element,), {"name": "lines"})
linestrips = type("linestrips", (Element,), {"name": "linestrips"})
link = type("link", (Element,), {"name": "link"})
linker = type("linker", (Element,), {"name": "linker"})
locked = type("locked", (Element,), {"name": "locked"})
logic_op_enable = type("logic_op_enable", (Element,), {"name": "logic_op_enable"})
logic_op = type("logic_op", (Element,), {"name": "logic_op"})
longitude = type("longitude", (Element,), {"name": "longitude"})
lookout = type("lookout", (Element,), {"name": "lookout"})
magfilter = type("magfilter", (Element,), {"name": "magfilter"})
mass_frame = type("mass_frame", (Element,), {"name": "mass_frame"})
rigid_body = type("rigid_body", (Element,), {"name": "rigid_body"})
mass = type("mass", (Element,), {"name": "mass"})
shape = type("shape", (Element,), {"name": "shape"})
material_ambient = type("material_ambient", (Element,), {"name": "material_ambient"})
material_diffuse = type("material_diffuse", (Element,), {"name": "material_diffuse"})
material_emission = type("material_emission", (Element,), {"name": "material_emission"})
material_shininess = type(
    "material_shininess", (Element,), {"name": "material_shininess"}
)
material_specular = type("material_specular", (Element,), {"name": "material_specular"})
material = type("material", (Element,), {"name": "material"})
matrix = type("matrix", (Element,), {"name": "matrix"})
max_anisotropy = type("max_anisotropy", (Element,), {"name": "max_anisotropy"})
mesh = type("mesh", (Element,), {"name": "mesh"})
minfilter = type("minfilter", (Element,), {"name": "minfilter"})
mip_bias = type("mip_bias", (Element,), {"name": "mip_bias"})
mip_max_level = type("mip_max_level", (Element,), {"name": "mip_max_level"})
mip_min_level = type("mip_min_level", (Element,), {"name": "mip_min_level"})
mipfilter = type("mipfilter", (Element,), {"name": "mipfilter"})
mips = type("mips", (Element,), {"name": "mips"})
create_cube = type("create_cube", (Element,), {"name": "create_cube"})
create2d = type("create2d", (Element,), {"name": "create2d"})
create3d = type("create3d", (Element,), {"name": "create3d"})
model_view_matrix = type("model_view_matrix", (Element,), {"name": "model_view_matrix"})
modified = type("modified", (Element,), {"name": "modified"})
modifier = type("modifier", (Element,), {"name": "modifier"})
morph = type("morph", (Element,), {"name": "morph"})
motion = type("motion", (Element,), {"name": "motion"})
multisample_enable = type(
    "multisample_enable", (Element,), {"name": "multisample_enable"}
)
Name_array = type("Name_array", (Element,), {"name": "Name_array"})
emantic = type("emantic", (Element,), {"name": "emantic"})
newparam = type("newparam", (Element,), {"name": "newparam"})
ommon = type("ommon", (Element,), {"name": "ommon"})
node = type("node", (Element,), {"name": "node"})
normalize_enable = type("normalize_enable", (Element,), {"name": "normalize_enable"})
nurbs_surface = type("nurbs_surface", (Element,), {"name": "nurbs_surface"})
nurbs = type("nurbs", (Element,), {"name": "nurbs"})
optics = type("optics", (Element,), {"name": "optics"})
orient = type("orient", (Element,), {"name": "orient"})
origin = type("origin", (Element,), {"name": "origin"})
orthographic = type("orthographic", (Element,), {"name": "orthographic"})
p = type("p", (Element,), {"name": "p"})
edges = type("edges", (Element,), {"name": "edges"})
faces = type("faces", (Element,), {"name": "faces"})
lines = type("lines", (Element,), {"name": "lines"})
pcurves = type("pcurves", (Element,), {"name": "pcurves"})
ph = type("ph", (Element,), {"name": "ph"})
solids = type("solids", (Element,), {"name": "solids"})
triangles = type("triangles", (Element,), {"name": "triangles"})
trifans = type("trifans", (Element,), {"name": "trifans"})
tristrips = type("tristrips", (Element,), {"name": "tristrips"})
wires = type("wires", (Element,), {"name": "wires"})
parabola = type("parabola", (Element,), {"name": "parabola"})
param = type("param", (Element,), {"name": "param"})
ph = type("ph", (Element,), {"name": "ph"})
phong = type("phong", (Element,), {"name": "phong"})
physics_material = type("physics_material", (Element,), {"name": "physics_material"})
physics_model = type("physics_model", (Element,), {"name": "physics_model"})
physics_scene = type("physics_scene", (Element,), {"name": "physics_scene"})
plane = type("plane", (Element,), {"name": "plane"})
point_distance_attenuation = type(
    "point_distance_attenuation", (Element,), {"name": "point_distance_attenuation"}
)
point_fade_threshold_size = type(
    "point_fade_threshold_size", (Element,), {"name": "point_fade_threshold_size"}
)
point_size_max = type("point_size_max", (Element,), {"name": "point_size_max"})
point_size_min = type("point_size_min", (Element,), {"name": "point_size_min"})
point_size = type("point_size", (Element,), {"name": "point_size"})
point_smooth_enable = type(
    "point_smooth_enable", (Element,), {"name": "point_smooth_enable"}
)
polygon_mode = type("polygon_mode", (Element,), {"name": "polygon_mode"})
polygon_offset_fill_enable = type(
    "polygon_offset_fill_enable", (Element,), {"name": "polygon_offset_fill_enable"}
)
polygon_offset_line_enable = type(
    "polygon_offset_line_enable", (Element,), {"name": "polygon_offset_line_enable"}
)
polygon_offset_point_enable = type(
    "polygon_offset_point_enable", (Element,), {"name": "polygon_offset_point_enable"}
)
polygon_offset = type("polygon_offset", (Element,), {"name": "polygon_offset"})
polygon_smooth_enable = type(
    "polygon_smooth_enable", (Element,), {"name": "polygon_smooth_enable"}
)
polygon_stipple_enable = type(
    "polygon_stipple_enable", (Element,), {"name": "polygon_stipple_enable"}
)
polygons = type("polygons", (Element,), {"name": "polygons"})
polylist = type("polylist", (Element,), {"name": "polylist"})
prismatic = type("prismatic", (Element,), {"name": "prismatic"})
profile_BRIDGE = type("profile_BRIDGE", (Element,), {"name": "profile_BRIDGE"})
profile_CG = type("profile_CG", (Element,), {"name": "profile_CG"})
profile_COMMON = type("profile_COMMON", (Element,), {"name": "profile_COMMON"})
verview = type("verview", (Element,), {"name": "verview"})
profile_GLES = type("profile_GLES", (Element,), {"name": "profile_GLES"})
profile_GLES2 = type("profile_GLES2", (Element,), {"name": "profile_GLES2"})
profile_GLSL = type("profile_GLSL", (Element,), {"name": "profile_GLSL"})
program = type("program", (Element,), {"name": "program"})
projection_matrix = type("projection_matrix", (Element,), {"name": "projection_matrix"})
quadratic_attenuation = type(
    "quadratic_attenuation", (Element,), {"name": "quadratic_attenuation"}
)
point = type("point", (Element,), {"name": "point"})
radius = type("radius", (Element,), {"name": "radius"})
capsule = type("capsule", (Element,), {"name": "capsule"})
circle = type("circle", (Element,), {"name": "circle"})
cone = type("cone", (Element,), {"name": "cone"})
cylinder = type("cylinder", (Element,), {"name": "cylinder"})
ellipse = type("ellipse", (Element,), {"name": "ellipse"})
hyperbola = type("hyperbola", (Element,), {"name": "hyperbola"})
torus = type("torus", (Element,), {"name": "torus"})
ref_attachment = type("ref_attachment", (Element,), {"name": "ref_attachment"})
ref = type("ref", (Element,), {"name": "ref"})
binary = type("binary", (Element,), {"name": "binary"})
init_from = type("init_from", (Element,), {"name": "init_from"})
reflective = type("reflective", (Element,), {"name": "reflective"})
reflectivity = type("reflectivity", (Element,), {"name": "reflectivity"})
render = type("render", (Element,), {"name": "render"})
renderable = type("renderable", (Element,), {"name": "renderable"})
rescale_normal_enable = type(
    "rescale_normal_enable", (Element,), {"name": "rescale_normal_enable"}
)
restitution = type("restitution", (Element,), {"name": "restitution"})
revision = type("revision", (Element,), {"name": "revision"})
revolute = type("revolute", (Element,), {"name": "revolute"})
RGB = type("RGB", (Element,), {"name": "RGB"})
rigid_constraint = type("rigid_constraint", (Element,), {"name": "rigid_constraint"})
rotate = type("rotate", (Element,), {"name": "rotate"})
sample_alpha_to_coverage_enable = type(
    "sample_alpha_to_coverage_enable",
    (Element,),
    {"name": "sample_alpha_to_coverage_enable"},
)
sample_alpha_to_one_enable = type(
    "sample_alpha_to_one_enable", (Element,), {"name": "sample_alpha_to_one_enable"}
)
sample_coverage_enable = type(
    "sample_coverage_enable", (Element,), {"name": "sample_coverage_enable"}
)
sample_coverage = type("sample_coverage", (Element,), {"name": "sample_coverage"})
sampler_image = type("sampler_image", (Element,), {"name": "sampler_image"})
sampler_states = type("sampler_states", (Element,), {"name": "sampler_states"})
sampler = type("sampler", (Element,), {"name": "sampler"})
nterpolation = type("nterpolation", (Element,), {"name": "nterpolation"})
sampler1D = type("sampler1D", (Element,), {"name": "sampler1D"})
sampler2D = type("sampler2D", (Element,), {"name": "sampler2D"})
sampler3D = type("sampler3D", (Element,), {"name": "sampler3D"})
samplerCUBE = type("samplerCUBE", (Element,), {"name": "samplerCUBE"})
samplerDEPTH = type("samplerDEPTH", (Element,), {"name": "samplerDEPTH"})
samplerRECT = type("samplerRECT", (Element,), {"name": "samplerRECT"})
scale = type("scale", (Element,), {"name": "scale"})
scene = type("scene", (Element,), {"name": "scene"})
scissor_test_enable = type(
    "scissor_test_enable", (Element,), {"name": "scissor_test_enable"}
)
scissor = type("scissor", (Element,), {"name": "scissor"})
semantic = type("semantic", (Element,), {"name": "semantic"})
setparam = type("setparam", (Element,), {"name": "setparam"})
shade_model = type("shade_model", (Element,), {"name": "shade_model"})
shader = type("shader", (Element,), {"name": "shader"})
shape = type("shape", (Element,), {"name": "shape"})
shells = type("shells", (Element,), {"name": "shells"})
shininess = type("shininess", (Element,), {"name": "shininess"})
SIDREF_array = type("SIDREF_array", (Element,), {"name": "SIDREF_array"})
size_exact = type("size_exact", (Element,), {"name": "size_exact"})
size_ratio = type("size_ratio", (Element,), {"name": "size_ratio"})
size = type("size", (Element,), {"name": "size"})
create_cube = type("create_cube", (Element,), {"name": "create_cube"})
create3d = type("create3d", (Element,), {"name": "create3d"})
skeleton = type("skeleton", (Element,), {"name": "skeleton"})
skew = type("skew", (Element,), {"name": "skew"})
skin = type("skin", (Element,), {"name": "skin"})
source_data = type("source_data", (Element,), {"name": "source_data"})
source = type("source", (Element,), {"name": "source"})
sources = type("sources", (Element,), {"name": "sources"})
specular = type("specular", (Element,), {"name": "specular"})
speed = type("speed", (Element,), {"name": "speed"})
sphere = type("sphere", (Element,), {"name": "sphere"})
spline = type("spline", (Element,), {"name": "spline"})
nterpolation = type("nterpolation", (Element,), {"name": "nterpolation"})
spot = type("spot", (Element,), {"name": "spot"})
spring = type("spring", (Element,), {"name": "spring"})
states = type("states", (Element,), {"name": "states"})
static_friction = type("static_friction", (Element,), {"name": "static_friction"})
stencil_clear = type("stencil_clear", (Element,), {"name": "stencil_clear"})
stencil_func_separate = type(
    "stencil_func_separate", (Element,), {"name": "stencil_func_separate"}
)
stencil_func = type("stencil_func", (Element,), {"name": "stencil_func"})
stencil_mask_separate = type(
    "stencil_mask_separate", (Element,), {"name": "stencil_mask_separate"}
)
stencil_mask = type("stencil_mask", (Element,), {"name": "stencil_mask"})
stencil_op_separate = type(
    "stencil_op_separate", (Element,), {"name": "stencil_op_separate"}
)
stencil_op = type("stencil_op", (Element,), {"name": "stencil_op"})
stencil_target = type("stencil_target", (Element,), {"name": "stencil_target"})
stencil_test_enable = type(
    "stencil_test_enable", (Element,), {"name": "stencil_test_enable"}
)
stiffness = type("stiffness", (Element,), {"name": "stiffness"})
subject = type("subject", (Element,), {"name": "subject"})
surface_curves = type("surface_curves", (Element,), {"name": "surface_curves"})
surface = type("surface", (Element,), {"name": "surface"})
surfaces = type("surfaces", (Element,), {"name": "surfaces"})
swept_surface = type("swept_surface", (Element,), {"name": "swept_surface"})
swing_cone_and_twist = type(
    "swing_cone_and_twist", (Element,), {"name": "swing_cone_and_twist"}
)
target_value = type("target_value", (Element,), {"name": "target_value"})
target = type("target", (Element,), {"name": "target"})
targets = type("targets", (Element,), {"name": "targets"})
bind_material = type("bind_material", (Element,), {"name": "bind_material"})
formula = type("formula", (Element,), {"name": "formula"})
instance_rigid_body = type(
    "instance_rigid_body", (Element,), {"name": "instance_rigid_body"}
)
kinematics_model = type("kinematics_model", (Element,), {"name": "kinematics_model"})
kinematics = type("kinematics", (Element,), {"name": "kinematics"})
light = type("light", (Element,), {"name": "light"})
motion = type("motion", (Element,), {"name": "motion"})
optics = type("optics", (Element,), {"name": "optics"})
rigid_body = type("rigid_body", (Element,), {"name": "rigid_body"})
verview = type("verview", (Element,), {"name": "verview"})
technique_hint = type("technique_hint", (Element,), {"name": "technique_hint"})
technique_override = type(
    "technique_override", (Element,), {"name": "technique_override"}
)
texcombiner = type("texcombiner", (Element,), {"name": "texcombiner"})
texcoord = type("texcoord", (Element,), {"name": "texcoord"})
texenv = type("texenv", (Element,), {"name": "texenv"})
texture_env_color = type("texture_env_color", (Element,), {"name": "texture_env_color"})
texture_env_mode = type("texture_env_mode", (Element,), {"name": "texture_env_mode"})
texture_pipeline = type("texture_pipeline", (Element,), {"name": "texture_pipeline"})
texture = type("texture", (Element,), {"name": "texture"})
texture1D_enable = type("texture1D_enable", (Element,), {"name": "texture1D_enable"})
texture1D = type("texture1D", (Element,), {"name": "texture1D"})
texture2D_enable = type("texture2D_enable", (Element,), {"name": "texture2D_enable"})
texture2D = type("texture2D", (Element,), {"name": "texture2D"})
texture3D_enable = type("texture3D_enable", (Element,), {"name": "texture3D_enable"})
texture3D = type("texture3D", (Element,), {"name": "texture3D"})
textureCUBE_enable = type(
    "textureCUBE_enable", (Element,), {"name": "textureCUBE_enable"}
)
textureCUBE = type("textureCUBE", (Element,), {"name": "textureCUBE"})
textureDEPTH_enable = type(
    "textureDEPTH_enable", (Element,), {"name": "textureDEPTH_enable"}
)
textureDEPTH = type("textureDEPTH", (Element,), {"name": "textureDEPTH"})
textureRECT_enable = type(
    "textureRECT_enable", (Element,), {"name": "textureRECT_enable"}
)
textureRECT = type("textureRECT", (Element,), {"name": "textureRECT"})
time_step = type("time_step", (Element,), {"name": "time_step"})
title = type("title", (Element,), {"name": "title"})
translate = type("translate", (Element,), {"name": "translate"})
transparency = type("transparency", (Element,), {"name": "transparency"})
transparent = type("transparent", (Element,), {"name": "transparent"})
unit = type("unit", (Element,), {"name": "unit"})
unnormalized = type("unnormalized", (Element,), {"name": "unnormalized"})
usertype = type("usertype", (Element,), {"name": "usertype"})
v = type("v", (Element,), {"name": "v"})
value = type("value", (Element,), {"name": "value"})
velocity = type("velocity", (Element,), {"name": "velocity"})
vertex_weights = type("vertex_weights", (Element,), {"name": "vertex_weights"})
wrap_p = type("wrap_p", (Element,), {"name": "wrap_p"})
wrap_s = type("wrap_s", (Element,), {"name": "wrap_s"})
wrap_t = type("wrap_t", (Element,), {"name": "wrap_t"})
xfov = type("xfov", (Element,), {"name": "xfov"})
xmag = type("xmag", (Element,), {"name": "xmag"})
yfov = type("yfov", (Element,), {"name": "yfov"})
ymag = type("ymag", (Element,), {"name": "ymag"})
zfar = type("zfar", (Element,), {"name": "zfar"})
znear = type("znear", (Element,), {"name": "znear"})

# builtins require an underscore.
_input = type("input", (closed_tag,), {"name": "input"})
_float = type("float", (Element,), {"name": "float"})
_pass = type("pass", (Element,), {"name": "pass"})
_max = type("max", (Element,), {"name": "max"})
_min = type("min", (Element,), {"name": "min"})
_import = type("import", (Element,), {"name": "import"})
_hex = type("hex", (Element,), {"name": "hex"})
_format = type("format", (Element,), {"name": "format"})


class xml_header:
    """xml

    Returns:
        str: <?xml version="1.0"?>
    """

    def __str__(self):
        return '<?xml version="1.0"?>'


"""

# The asset tag describes the author and environment of the file
TMP_ASSET = asset(
    author("alorino"),
    up_axis("Y_UP")
)

# common cameras are perspective or orthographic
LIB_CAMERAS = library_cameras(
    camera(_id="PerspCamera", _name="PerspCamera").appendChild(
        optics(
            technique_common(
                perspective(
                    yfov(37.8493),
                    aspect_ratio(1),
                    znear(10),
                    zfar(1000)
                )
            )
        )
    )
)

# Common lights are point, spot or directional
LIB_LIGHTS = library_lights(
    light(_id="pointLightShape1-lib", _name="pointLightShape1").appendChild(
        technique_common(
            point(
                color("1 1 1"),  # need to figure this out
                constant_attenuation(1),
                linear_attenuation(0),
                quadratic_attenuation(0)
            )
        )
    )
)

# Materials instance effects
LIB_MAT = library_materials(
    material(_id="Blue", _name="Blue").appendChild(
        instance_effect(_url="#Blue-fx")
    )
)

# Common effects act like the OpenGL 1 state
LIB_FX = library_effects(
    effect(_id="Blue-fx").appendChild(
        profile_COMMON(
            technique(_sid="common").appendChild(
                phong(
                    emission(
                        color("0 0 0 1"),
                        index_of_refraction(
                            _float(0)
                        )
                    )
                )
            )
        )
    )
)

# Geometry describes the OpenGL attributes
LIB_GEOM = library_geometries(
    geometry(_id="box-lib", _name="box").appendChild(
        mesh(
            source(_id="box-lib-positions", _name="position"),
            source(_id="box-lib-normals", _name="normal"),
            vertices(_id="box-lib-vertices").appendChild(
                _input(_semantic="POSITION", _source="#box-lib-positions")
            ),
            polylist(_count="6", _material="BlueSG").appendChild(
                _input(_offset="0", _semantic="VERTEX", _source="#box-lib-vertices"),
                _input(_offset="1", _semantic="NORMAL", _source="#box-lib-normals"),
                vcount("4 4 4 4 4 4"),
                p("0 0 2 1 3 2 1 3 0 4 1 5 5 6 4 7")
            )
        )
    )
)

# The scene tag specifies a visual scene and sometimes a phyiscs scene
LIB_SCENES = library_visual_scenes(
    visual_scene(_id="VisualSceneNode", _name="untitled").appendChild(
        node(_id="Camera", _name="Camera")
    )
)

TMP_SCENE = scene(
    instance_visual_scene(_url="#VisualSceneNode")
)

# Extras are a way of extending the format
TMP_EXTRA = extra(
    technique(_profile="steveT").appendChild(
        test_element(_id="my_test_element", _attr1="value1", _attr2="value2").appendChild(
            "this is some text"
        )
    )
)

"""

"""

library_cameras(
    camera(_id="Camera-camera", _name="Camera").html(
        optics(
            technique_common(
                perspective(
                    xfov("49.13434", _sid="xfov")
                    aspect_ratio("1.777778")  # NOTE - underscored attributes?. - likely to break?
                    znear("0.1", _sid="znear")
                    zfar("100", _sid="zfar")
                )
            )
        )
    )
)

source(_id="Cube-mesh-normals").html(
    float_array(_id="Cube-mesh-normals-array", _count="18").html("0 0 -1 0 0 1 1 -2.83e-7 0 -2.83e-7 -1 0 -1 2.23e-7 -1.34e-7 2.38e-7 1 2.08e-7"),
        technique_common(
            accessor(_source="#Cube-mesh-normals-array", _count="6", _stride="3").html(
                param(_name="X" type="float"),
                param(_name="Y" type="float"),
            param(_name="Z" type="float")
        )
    )
)

vertices(_id="Cube-mesh-vertices").html(
    input(_semantic="POSITION", _source="#Cube-mesh-positions")
)


COLLADA(
    asset(
        created("2005-06-27T21:00:00Z"),
        keywords("COLLADA inthange"),
        created("2005-06-27T21:00:00Z"),
        unit(_name="nautical_league", _meter="5556.0"),
        up_axis("Z_UP")
    )
)

light(_id="blue").html(
    technique_common(
        directional(
        color(0.1 0.1 0.5)
        )
    )
)


camera(_name="eyepoint").html(
    optics(
        technique_common("....")
        technique profile="MyFancyGIRenderer">
            param("180.0", _name="FocalLength", _type="float")
            param("5.6", _name="Aperture", _type="float")
        )
    ),
    imager(
        technique(_profile="MyFancyGIRenderer").html(
            param("200.0", _name="ShutterSpeed", _type="float")
            param("0.2", _name="RedGain", _type="float")
            param("0.22", _name="GreenGain", _type="float")
            param("0.25", _name="BlueGain", _type="float")
            param("2.2", _name="RedGamma", _type="float")
            param("2.1", _name="GreenGamma", _type="float")
            param("2.17", _name="BlueGamma", _type="float")
            param("0.17", _name="BloomPixelLeak", _type="float")
            param("InvSquare", _name="BloomFalloff", _type="Name")
        )
    )
)


+ asset
+ + contributor
+ + + author
+ + + authoring_tool
+ + + created
+ + + unit
+ + + up_axi
+ library_cameras
+ + camera
+ + + optics
+ + + + technique_common
+ + + + + perspective
+ + + + + + xfov
+ + + + + + aspect_ratio
+ + + + + + znear
+ + + + + + zfar
+ library_lights
+ + light
+ + + technique_common
+ + + + point
+ + + + + color
+ + + + + constant_attenuation
+ + + + + linear_attenuation
+ + + + + quadratic_attenuation
+ + + extra
+ + + + technique
............
+ library_visual_scenes
+ + visual_scene
+ + + node
+ + + + translate
+ + + + rotate
+ + + + rotate
+ + + + rotate
+ + + + scale
+ + + + instance_geometry
+ + + + + bind_material
+ + + + + + technique_common
+ + + + + + + instance_material
+ + + node
+ + + + translate
+ + + + rotate
+ + + + rotate
+ + + + rotate
+ + + + scale
+ + + + instance_camera
+ scene
+ + instance_visual_scene

"""



"""
    domonic.i18n.fr
    ====================================

    html tags in French

    translated to French using google translate.
    send a pull request if you have a better translation.
"""
from domonic.html import *

# tags
corps = body
tete = tête = head
scenario = scénario = script
style = style
portail = portal
fort = strong
guillemets = blockquote
tableau = table
forme = form
etiquette = étiquette = label
soumettre = submit
titre = title
# noscript = noscript
rubrique = section
# nav = nav
article = article
decote = decôté = aside
hgroupe = hgroup
adresse = address
chiffre = figure
legendedelafigure = légendedelafigure = figcaption
petit = small
citer = cite
code = code
sous = sub
# sup = sup
marque = mark
rubis = ruby
envergure = span
# iframe = iframe
vidéo = video
audio = audio
toile = canvas
# legende = légende = caption
groupecol = colgroup
corps = tbody
# tête = thead
pied = tfoot
jeudechamps = fieldset
legende = légende = legend
bouton = button
selectionner = sélectionner = select
listededonnées = datalist
groupeopt = optgroup
option = option
zonedetexte = textarea
sortie = output
progres = progrès = progress
metre = mètre = meter
détails = details
resume = résumé = summary
# menu = menu
elementdemenu = élémentdemenu = menuitem
police = font
entete = entête = header
pieddepage = footer
base = base
lien = link
méta = meta
# img = img
incorporer = embed
source = source
piste = track
zone = area
# col = col
entree = entrée = input
commande = command
principal = main
policedebase = basefont
centre = center
incorporer = embed
# police = font
# isindex = isindex
liste = listing
texteenclair = plaintext
greve = grève = strike
modele = modèle = template
image = picture
typedoc = doctype
commentaire = comment



"""
    domonic.templates
    ====================================
    some builtin templates

"""

from domonic.constants import http_response_status_codes
from domonic.constants.entities import Char
from domonic.html import *


class status_page():
    DEBUG_MODE = False

    def __init__(self, code=404, wholepage=True):
        self.status_code = code
        self.message = http_response_status_codes[code]
        # self.host = "http://localhost:8000"
        self.wholepage = wholepage

        self.status_node = div(_id="status")
        self.status_node += h1(f"{self.status_code}")
        self.status_node += p(self.message)

        # if status_page.DEBUG_MODE:
        #     import os
        #     self.status_node.appendChild(
        #         pre(os.environ)
        #     )
        #     import sys
        #     self.status_node.appendChild(
        #         pre(sys.path)
        #     )
        #     import traceback
        #     self.status_node.appendChild(
        #         pre(traceback.format_exc())
        #     )

        page = html(
            head(title(f"{self.status_code}")),
            body(self.status_node)
        )
        self.content = self.wholepage and page or self.status_node

    # def __getattr__(self, name):
    #     return getattr(self.status_node, name)

    # def __setattr__(self, name, value):
    #     setattr(self.status_node, name, value)

    def __str__(self):
        return f'{self.content}'

    # @staticmethod
    # def to_html(output_dir):
    #     for code, name in status_page.STATUS.items():
    #         page = status_page(code, name)
    #         with open(f"{output_dir}/{code}.html", "w") as f:
    #             f.write(f'{page}')


'''
class d_b():
    def __init__(self):
        pass
    def config(self, db_name, db_user, db_pass, db_host, db_port):
        self.db_name = db_name
        self.db_user = db_user
        self.db_pass = db_pass
        self.db_host = db_host
        self.db_port = db_port
    # def connect(self):
    # def query(self, query):
    # def close(self):
'''

# from domonic.templates import status_page

# @app.errorhandler(404)
# def page_not_found(e):
#     return status_page(404)



"""
    domonic.webapi.encoding
    ====================================
    https://developer.mozilla.org/en-US/docs/Web/API/Encoding_API
"""

from domonic.javascript import (ArrayBuffer, DataView, Float32Array,
                                Float64Array, Function, Int8Array, Int16Array,
                                Int32Array, String, Uint8Array, Uint16Array,
                                Uint32Array)
from domonic.webapi.streams import ReadableStream

# from domonic.javascript import ArrayBufferView


class TextDecoder:
    def __init__(self, encoding="utf-8"):
        self.encoding = encoding

    def decode(self, bytes_or_buffer):
        if self.encoding == "utf-8":
            # get the bytes from the buffer
            if isinstance(bytes_or_buffer, ReadableStream):
                bytes_or_buffer = bytes_or_buffer.read()
            if isinstance(bytes_or_buffer, ArrayBuffer):
                bytes_or_buffer = bytes_or_buffer.buffer
            if isinstance(bytes_or_buffer, (Uint8Array, Int8Array, Uint16Array, Int16Array, Int32Array, Uint32Array, Float32Array, Float64Array, DataView)):
                bytes_or_buffer = bytes_or_buffer.buffer
            # if isinstance(bytes_or_buffer, ArrayBufferView):

            # convert the bytes to a string
            # return bytes_or_buffer.decode("utf-8")
            # return "".join([b.decode('UTF-8') for b in bytes_or_buffer])
            b = bytearray(bytes_or_buffer)
            return b.decode("utf-8")
            # return b.decode("utf-16")

    # def __repr__(self):
    #     return f"<TextDecoder encoding={self.encoding}>"


class TextEncoder:
    def __init__(self, encoding="utf-8"):
        self.encoding = encoding

    def encode(self, string):
        return string.encode(self.encoding)

    def encodeInto(self, string, bytes_or_buffer):
        return bytes_or_buffer.encode(self.encoding)

    def __repr__(self):
        return f"<TextEncoder encoding={self.encoding}>"


class TextDecoderStream(ReadableStream):
    def __init__(self, encoding="utf-8"):
        super().__init__()
        # self.decoder = TextDecoder(encoding)
        self.encoding = encoding
        self.readable = True
        self.writable = False
        self.ignoreBOM = False


    def read(self, size=None):
        return self.decoder.decode(super().read(size))

    def __repr__(self):
        return f"<TextDecoderStream encoding={self.decoder.encoding}>"


class TextEncoderStream(ReadableStream):
    def __init__(self, encoding="utf-8"):
        super().__init__()
        # self.encoder = TextEncoder(encoding)
        self.encoding = encoding
        self.readable = True
        self.writable = False

    def read(self, size=None):
        return self.encoder.encode(super().read(size))

    def __repr__(self):
        return f"<TextEncoderStream encoding={self.encoder.encoding}>"


"""
    domonic.svg
    ====================================
    Generate SVG with python 3

    WARNING - totally not tested. except circle. I just assumed it would work...

    # https://www.w3.org/TR/SVG2/eltindex.html

"""
from domonic.html import a, audio, canvas, video, script, style, iframe
from domonic.dom import Element

svg_tags = [
    "svg",
    "animate",
    "animateMotion",
    "animateTransform",
    "circle",
    "clipPath",
    "defs",
    "desc",
    "ellipse",
    "image",
    "line",
    "linearGradient",
    "marker",
    "mask",
    "mpath",
    "pattern",
    "polygon",
    "polyline",
    "radialGradient",
    "tspan",
    "path",
    "rect",
    "stop",
    "switch",
    "symbol",
    "text",
    "textPath",
    "title",
    "use",
    "view",
    "feBlend",
    "feColorMatrix",
    "feComponentTransfer",
    "feComposite",
    "feConvolveMatrix",
    "feDiffuseLighting",
    "feDisplacementMap",
    "feGaussianBlur",
    "feImage",
    "feMerge",
    "feMorphology",
    "feOffset",
    "feSpecularLighting",
    "feTile",
    "feTurbulence",
    "feDistantLight",
    "fePointLight",
    "feSpotLight",
    "feDropShadow",
    "discard",
    "feDistantLight",
    "feFlood",
    "feFuncA",
    "feFuncB",
    "feFuncG",
    "feFuncR",
    "feMergeNode",
    "foreignObject",
    "unknown",
    "cursor",
    "hatchpath",
    "altGlyph",
    "tref",
    "tspan",
    "altGlyphDef",
    "altGlyphItem",
    "glyph",
    "glyphRef",
    "solidcolor",
    "hatch",
    "font",
    "hkern",
    "vkern",
    "animateColor",
    "g",
]


# def svg_init(self, *args, **kwargs):
#     tag.__init__(self, *args, **kwargs)
#     Element.__init__(self, *args, **kwargs)


svg = type('svg', (Element,), {'name': 'svg'})

animate = type('animate', (Element,), {'name': 'animate'})
animateMotion = type('animateMotion', (Element,), {'name': 'animateMotion'})
animateTransform = type('animateTransform', (Element,), {'name': 'animateTransform'})
circle = type('circle', (Element,), {'name': 'circle'})
clipPath = type('clipPath', (Element,), {'name': 'clipPath'})
defs = type('defs', (Element,), {'name': 'defs'})
desc = type('desc', (Element,), {'name': 'desc'})
ellipse = type('ellipse', (Element,), {'name': 'ellipse'})
g = type('g', (Element,), {'name': 'g'})
image = type('image', (Element,), {'name': 'image'})
line = type('line', (Element,), {'name': 'line'})
linearGradient = type('linearGradient', (Element,), {'name': 'linearGradient'})
marker = type('marker', (Element,), {'name': 'marker'})
mask = type('mask', (Element,), {'name': 'mask'})
mpath = type('mpath', (Element,), {'name': 'mpath'})
pattern = type('pattern', (Element,), {'name': 'pattern'})
polygon = type('polygon', (Element,), {'name': 'polygon'})
polyline = type('polyline', (Element,), {'name': 'polyline'})
radialGradient = type('radialGradient', (Element,), {'name': 'radialGradient'})

tspan = type('tspan', (Element,), {'name': 'tspan'})
path = type('path', (Element,), {'name': 'path'})
rect = type('rect', (Element,), {'name': 'rect'})
stop = type('stop', (Element,), {'name': 'stop'})
switch = type('switch', (Element,), {'name': 'switch'})
symbol = type('symbol', (Element,), {'name': 'symbol'})
text = type('text', (Element,), {'name': 'text'})
textPath = type('textPath', (Element,), {'name': 'textPath'})
title = type('title', (Element,), {'name': 'title'})
use = type('use', (Element,), {'name': 'use'})
view = type('view', (Element,), {'name': 'view'})

feBlend = type('feBlend', (Element,), {'name': 'feBlend'})
feColorMatrix = type('feColorMatrix', (Element,), {'name': 'feColorMatrix'})
feComponentTransfer = type('feComponentTransfer', (Element,), {'name': 'feComponentTransfer'})
feComposite = type('feComposite', (Element,), {'name': 'feComposite'})
feConvolveMatrix = type('feConvolveMatrix', (Element,), {'name': 'feConvolveMatrix'})
feDiffuseLighting = type('feDiffuseLighting', (Element,), {'name': 'feDiffuseLighting'})
feDisplacementMap = type('feDisplacementMap', (Element,), {'name': 'feDisplacementMap'})
feGaussianBlur = type('feGaussianBlur', (Element,), {'name': 'feGaussianBlur'})
feImage = type('feImage', (Element,), {'name': 'feImage'})
feMerge = type('feMerge', (Element,), {'name': 'feMerge'})
feMorphology = type('feMorphology', (Element,), {'name': 'feMorphology'})
feOffset = type('feOffset', (Element,), {'name': 'feOffset'})
feSpecularLighting = type('feSpecularLighting', (Element,), {'name': 'feSpecularLighting'})
feTile = type('feTile', (Element,), {'name': 'feTile'})
feTurbulence = type('feTurbulence', (Element,), {'name': 'feTurbulence'})
feDistantLight = type('feDistantLight', (Element,), {'name': 'feDistantLight'})
fePointLight = type('fePointLight', (Element,), {'name': 'fePointLight'})
feSpotLight = type('feSpotLight', (Element,), {'name': 'feSpotLight'})

feDropShadow = type('feDropShadow', (Element,), {'name': 'feDropShadow'})
discard = type('discard', (Element,), {'name': 'discard'})
feDistantLight = type('feDistantLight', (Element,), {'name': 'feDistantLight'})
feFlood = type('feFlood', (Element,), {'name': 'feFlood'})
feFuncA = type('feFuncA', (Element,), {'name': 'feFuncA'})
feFuncB = type('feFuncB', (Element,), {'name': 'feFuncB'})
feFuncG = type('feFuncG', (Element,), {'name': 'feFuncG'})
feFuncR = type('feFuncR', (Element,), {'name': 'feFuncR'})
feMergeNode = type('feMergeNode', (Element,), {'name': 'feMergeNode'})
foreignObject = type('foreignObject', (Element,), {'name': 'foreignObject'})
unknown = type('unknown', (Element,), {'name': 'unknown'})

cursor = type('cursor', (Element,), {'name': 'cursor'})
hatchpath = type('hatchpath', (Element,), {'name': 'hatchpath'})
altGlyph = type('altGlyph', (Element,), {'name': 'altGlyph'})
tref = type('tref', (Element,), {'name': 'tref'})
tspan = type('tspan', (Element,), {'name': 'tspan'})
altGlyphDef = type('altGlyphDef', (Element,), {'name': 'altGlyphDef'})
altGlyphItem = type('altGlyphItem', (Element,), {'name': 'altGlyphItem'})
glyph = type('glyph', (Element,), {'name': 'glyph'})
glyphRef = type('glyphRef', (Element,), {'name': 'glyphRef'})
solidcolor = type('solidcolor', (Element,), {'name': 'solidcolor'})
hatch = type('hatch', (Element,), {'name': 'hatch'})
font = type('font', (Element,), {'name': 'font'})
hkern = type('hkern', (Element,), {'name': 'hkern'})
vkern = type('vkern', (Element,), {'name': 'vkern'})
animateColor = type('animateColor', (Element,), {'name': 'animateColor'})
metadata = type('metadata', (Element,), {'name': 'metadata'})

# TODO --
# _filter # builtin
# _set # builtin

# are these obs or new?
# meshgradient 🗑️?
# mesh 🗑️?
# meshpatch 🗑️?
# meshrow 🗑️?

# TODO - just use underscores for these tags?
# missing-glyph
# color-profile
# font-face
# font-face-format
# font-face-name
# font-face-src
# font-face-uri



"""
    domonic.d3
    ====================================

"""

# from domonic.d3.queue import *
# from domonic.d3.array import *
# from domonic.d3.axis import *
# from domonic.d3.brush import *
# from domonic.d3.chord import *
# from domonic.d3.color import *
# from domonic.d3.contour import *
# from domonic.d3.delaunay import *
from domonic.d3.dispatch import *
from domonic.d3.format import *
# from domonic.d3.drag import *
# from domonic.d3.dsv import *
# from domonic.d3.ease import *
# from domonic.d3.fetch import *
# from domonic.d3.force import *
# from domonic.d3.format import *
# from domonic.d3.geo import *
# from domonic.d3.heirarchy import *
# from domonic.d3.interpolate import *
from domonic.d3.path import *
from domonic.d3.polygon import *
# from domonic.d3.quadtree import *
# from domonic.d3.queue import *
# from domonic.d3.random import *
# from domonic.d3.scale-chromatic import *
# from domonic.d3.scale import *
from domonic.d3.selection import *
from domonic.dom import document
from domonic.html import *

# from domonic.d3.shape import *
# from domonic.d3.tile import *
# from domonic.d3.time-format import *
# from domonic.d3.time import *
# from domonic.d3.timer import *
# from domonic.d3.transition import *
# from domonic.d3.zoom import *
